@@ -13,7 +13,6 @@ error = hPutStr stdout ==> putStr
 error = hPutStrLn stdout ==> putStrLn
 error = hPrint stdout ==> print
 
-
 -- ORD
 
 error = not (a == b) ==> a /= b
@@ -28,7 +27,8 @@ error = compare x y /= LT ==> x >= y
 error = compare x y == GT ==> x > y
 error = compare (f x) (f y) ==> Data.Ord.comparing f x y
 error = on compare f ==> Data.Ord.comparing f
-
+error = x == y || x == z ==> x `elem` [y,z]
+error = x /= y || x /= z ==> x `notElem` [y,z]
 
 -- READ/SHOW
 
@@ -39,6 +39,11 @@ error = showsPrec 0 ==> shows
 -- LIST
 
 error = concat (map f x) ==> concatMap f x
+warn = concat [a,b] ==> a ++ b
+-- these hints pick up way too much
+--warn = a++b++c++d++e++f++g ==> concat [a, b, c, d, e, f, g]
+--warn = a++b++c++d++e++f ==> concat [a, b, c, d, e, f]
+--warn = a++b++c++d++e ==> concat [a, b, c, d, e]
 error "Use map once" = map f (map g x) ==> map (f . g) x
 warn  = x !! 0 ==> head x
 error = take n (repeat x) ==> replicate n x
@@ -48,7 +53,7 @@ error = head (reverse x) ==> last x
 error = head (drop n x) ==> x !! n
 error = reverse (tail (reverse x)) ==> init x
 error = isPrefixOf (reverse x) (reverse y) ==> isSuffixOf x y
-error = foldr (++) [] x ==> concat x
+error = foldr (++) [] ==> concat
 error = span (not . p) ==> break p
 error = break (not . p) ==> span p
 error = concatMap (++ "\n") ==> unlines
@@ -61,14 +66,17 @@ warn  "Use null" = length x /= 0 ==> not (null x)
 error "Use :" = (\x -> [x]) ==> (:[])
 error = map (uncurry f) (zip x y) ==> zipWith f x y
 error = not (elem x y) ==> notElem x y
+warn  = foldr f z (map g x) ==> foldr (f . g) z x
 
 -- FOLDS
 
 error = foldr (&&) True ==> and
-error = foldr (>>) (return ()) ==> sequence_
+error = foldl (&&) True ==> and
 error = foldr (||) False ==> or
+error = foldl (||) False ==> or
+error = foldr (>>) (return ()) ==> sequence_
 error = foldl (+) 0 ==> sum
-warn  = foldl (*) 1 ==> product
+error = foldl (*) 1 ==> product
 
 -- FUNCTION
 
@@ -77,7 +85,10 @@ error = (\(_,y) -> y) ==> snd
 error = (\(x,_) -> x) ==> fst
 error = (\x y-> f (x,y)) ==> curry f where _ = notIn [x,y] f
 error = (\(x,y) -> f x y) ==> uncurry f where _ = notIn [x,y] f
-warn  = (\x -> f x y) ==> flip f y where _ = notIn x [f,y]
+error "Redundant $" = (($) . f) ==> f
+error "Redundant $" = (f $) ==> f
+warn  = (\x -> y) ==> const y where _ = isAtom y && notIn x y
+error "Redundant flip" = flip f x y ==> f y x where _ = isApp original
 error "Redundant id" = id x ==> x
 error "Redundant const" = const x y ==> x
 
@@ -89,11 +100,12 @@ error "Redundant if" = (if a then True else False) ==> a
 error "Redundant if" = (if a then False else True) ==> not a
 error "Redundant if" = (if a then t else (if b then t else f)) ==> if a || b then t else f
 error "Redundant if" = (if a then (if b then t else f) else f) ==> if a && b then t else f
-error "Redundant if" = (if x then True else y) ==> x || y
-error "Redundant if" = (if x then y else False) ==> x && y
-error "Use if" = case a of {True -> t; False -> f} ==> if a then t else f
-error "Use if" = case a of {True -> t; _ -> f} ==> if a then t else f
-error "Use if" = case a of {False -> f; _ -> t} ==> if a then t else f
+error "Redundant if" = (if x then True else y) ==> x || y where _ = notEq y False
+error "Redundant if" = (if x then y else False) ==> x && y where _ = notEq y True
+warn  "Use if" = case a of {True -> t; False -> f} ==> if a then t else f
+warn  "Use if" = case a of {False -> f; True -> t} ==> if a then t else f
+warn  "Use if" = case a of {True -> t; _ -> f} ==> if a then t else f
+warn  "Use if" = case a of {False -> f; _ -> t} ==> if a then t else f
 
 -- ARROW
 
@@ -145,10 +157,30 @@ error "Redundant $!" = id $! x ==> x
 
 -- MAYBE
 
-error = maybe x id  ==> Data.Maybe.fromMaybe x
+error = maybe x id ==> Data.Maybe.fromMaybe x
 error = maybe False (const True) ==> Data.Maybe.isJust
 error = maybe True (const False) ==> Data.Maybe.isNothing
+error = not (isNothing x) ==> isJust x
+error = not (isJust x) ==> isNothing x
+error = maybe [] (:[]) ==> maybeToList
 error = catMaybes (map f x) ==> mapMaybe f x
+error = concatMap (maybeToList . f) ==> Data.Maybe.mapMaybe f
+error = concatMap maybeToList ==> catMaybes
+error = maybe n Just x ==> Control.Monad.mplus x n
+warn  = (case x of Nothing -> y; Just a -> a)  ==> fromMaybe y x
+warn  = (case x of Just a -> a; Nothing -> y)  ==> fromMaybe y x
+error = (if isNothing x then y else fromJust x) ==> fromMaybe y x
+error = (if isJust x then fromJust x else y) ==> fromMaybe y x
+
+-- INFIX
+
+warn "Use infix" = elem x y ==> x `elem` y where _ = not (isInfixApp original) && not (isParen result)
+warn "Use infix" = notElem x y ==> x `notElem` y where _ = not (isInfixApp original) && not (isParen result)
+warn "Use infix" = isInfixOf x y ==> x `isInfixOf` y where _ = not (isInfixApp original) && not (isParen result)
+warn "Use infix" = isSuffixOf x y ==> x `isSuffixOf` y where _ = not (isInfixApp original) && not (isParen result)
+warn "Use infix" = isPrefixOf x y ==> x `isPrefixOf` y where _ = not (isInfixApp original) && not (isParen result)
+warn "Use infix" = union x y ==> x `union` y where _ = not (isInfixApp original) && not (isParen result)
+warn "Use infix" = intersect x y ==> x `intersect` y where _ = not (isInfixApp original) && not (isParen result)
 
 -- MATHS
 
@@ -186,7 +218,7 @@ error "Evaluate" = length [] ==> 0
 error "Evaluate" = foldl f z [] ==> z
 error "Evaluate" = foldr f z [] ==> z
 error "Evaluate" = foldr1 f [x] ==> x
-error "Evaluate" = scanr f q0 [] ==> [q0]
+error "Evaluate" = scanr f z [] ==> [z]
 error "Evaluate" = scanr1 f [] ==> []
 error "Evaluate" = scanr1 f [x] ==> [x]
 error "Evaluate" = take n [] ==> []
@@ -200,15 +232,19 @@ error "Evaluate" = x - 0 ==> x
 error "Evaluate" = x * 1 ==> x
 error "Evaluate" = x / 1 ==> x
 error "Evaluate" = id x ==> x
+error "Evaluate" = concat [a] ==> a
+error "Evaluate" = concat [] ==> []
 
 -- COMPLEX
 
 error "Use isPrefixOf" = (take i s == t) ==> _eval_ ((i == length t) && (t `Data.List.isPrefixOf` s))
     where _ = (isList t || isLit t) && isLit i
 
+{-
+-- clever hint, but not actually a good idea
 warn  = (do a <- f; g a) ==> f >>= g
     where _ = (isAtom f || isApp f) && notIn a g
-
+-}
 
 
 {-
@@ -221,7 +257,6 @@ yes = f x where f x = concat . map head -- concatMap head
 yes = concat . map f . g -- concatMap f . g
 yes = concat $ map f x -- concatMap f x
 yes = "test" ++ concatMap (' ':) ["of","this"] -- unwords ("test":["of","this"])
-yes = concat . intersperse " " -- unwords
 yes = if f a then True else b -- f a || b
 yes = not (a == b) -- a /= b
 yes = not (a /= b) -- a == b
@@ -235,7 +270,6 @@ no  = take n (foo xs) == "hello"
 yes = head (reverse xs) -- last xs
 yes = reverse xs `isPrefixOf` reverse ys -- isSuffixOf xs ys
 no = putStrLn $ show (length xs) ++ "Test"
-yes = do line <- getLine; putStrLn line -- getLine >>= putStrLn 
 yes = ftable ++ map (\ (c, x) -> (toUpper c, urlEncode x)) ftable -- toUpper Control.Arrow.*** urlEncode
 yes = map (\(a,b) -> a) xs -- fst
 yes = map (\(a,_) -> a) xs -- fst
@@ -251,7 +285,27 @@ yes = map (uncurry (+)) $ zip [1 .. 5] [6 .. 10] -- zipWith (+) [1 .. 5] [6 .. 1
 no = do iter <- textBufferGetTextIter tb ; textBufferSelectRange tb iter iter
 no = flip f x $ \y -> y*y+y
 no = \x -> f x (g x)
-yes = \x -> f x (g y) -- flip f (g y)
+no = foo (\ v -> f v . g)
+yes = concat . intersperse " " -- unwords
+yes = Prelude.concat $ intersperse " " xs -- unwords xs
+yes = concat $ Data.List.intersperse " " xs -- unwords xs
+yes = if a then True else False -- a
+yes = if x then true else False -- x && true
+yes = elem x y -- x `elem` y
+yes = foo (elem x y) -- x `elem` y
+no  = x `elem` y
+no  = elem 1 [] : []
+test a = foo (\x -> True) -- const True
+h a = flip f x (y z) -- f (y z) x
+h a = flip f x $ y z
+yes x = case x of {True -> a ; False -> b} -- if x then a else b
+yes x = case x of {False -> a ; _ -> b} -- if x then b else a
+no = const . ok . toResponse $ "saved"
+yes = case x z of Nothing -> y z; Just pattern -> pattern -- fromMaybe (y z) (x z)
+yes = if p then s else return () -- Control.Monad.when p $ s
+error = a $$$$ b $$$$ c ==> a . b $$$$$ c
+yes = when (not . null $ asdf) -- unless (null asdf)
+
 
 import Prelude \
 yes = flip mapM -- Control.Monad.forM
