@@ -1,7 +1,7 @@
 
 module HLint.Generalise where
 
-warn = concatMap ==> (>>=)
+warn = concatMap ==> (=<<)
 warn = liftM ==> fmap
 warn = map ==> fmap
 warn = a ++ b ==> a `Data.Monoid.mappend` b
