@@ -1,7 +1,11 @@
 
-module Hint.All where
+module Hint.All(
+    Hint(..), DeclHint, ModuHint,
+    staticHints, dynamicHints
+    ) where
 
-import Type
+import Settings
+import Hint.Type
 
 import Hint.Match
 import Hint.List
@@ -33,7 +37,3 @@ staticHints =
 
 dynamicHints :: [Setting] -> Hint
 dynamicHints = DeclHint . readMatch
-
-
-allHints :: [Setting] -> [Hint]
-allHints xs = dynamicHints xs : map snd staticHints
