@@ -1,3 +1,4 @@
+{-# LANGUAGE FlexibleContexts #-}
 {-
     Suggest the use of camelCase
 
@@ -5,7 +6,8 @@
     _*[A-Za-z]*_?'?
 
     Apply this to things that would get exported by default only
-    Also disallow prop_ as it's a standard QuickCheck idiom
+    Also allow prop_ as it's a standard QuickCheck idiom
+    Also don't suggest anything mentioned elsewhere in the module
 
 <TEST>
 data Yes = Foo | Bar'Test -- data Yes = Foo | BarTest
@@ -16,43 +18,48 @@ data No = FOO | BarBAR | BarBBar
 yes_foo = yes_foo + yes_foo -- yesFoo = ...
 no = 1 where yes_foo = 2
 a -== b = 1
+myTest = 1; my_test = 1
 </TEST>
 -}
 
 
-module Hint.Naming where
+module Hint.Naming(namingHint) where
 
-import HSE.All
-import Type
+import Hint.Type
 import Data.List
 import Data.Char
 import Data.Maybe
+import Util
+import qualified Data.Set as Set
 
 
 namingHint :: DeclHint
-namingHint _ x = [warn "Use camelCase" (declSrcLoc x) x2 (replaceNames res x2) | not $ null res]
-    where res = [(n,y) | n <- nub $ getNames x, Just y <- [suggestName n]]
+namingHint _ modu = naming $ Set.fromList [x | Ident _ x <- universeS modu]
+
+naming :: Set.Set String -> Decl_ -> [Idea]
+naming seen x = [warn "Use camelCase" x2 (replaceNames res x2) | notNull res]
+    where res = [(n,y) | n <- nub $ getNames x, Just y <- [suggestName n], not $ y `Set.member` seen]
           x2 = shorten x
 
 
-shorten :: Decl -> Decl
+shorten :: Decl_ -> Decl_
 shorten x = case x of
-    FunBind (Match a b c d e _:_) -> FunBind [f (Match a b c d) e]
+    FunBind sl (Match a b c d _:_) -> FunBind sl [f (Match a b c) d]
     PatBind a b c d _ -> f (PatBind a b c) d
     x -> x
     where
-        dots = Var $ UnQual $ Ident "..." -- Must be an Ident, not a Symbol
-        f cont (UnGuardedRhs _) = cont (UnGuardedRhs dots) (BDecls [])
-        f cont (GuardedRhss _) = cont (GuardedRhss [GuardedRhs nullSrcLoc [Qualifier dots] dots]) (BDecls [])
+        dots = Var an $ UnQual an $ Ident an "..." -- Must be an Ident, not a Symbol
+        f cont (UnGuardedRhs _ _) = cont (UnGuardedRhs an dots) Nothing
+        f cont (GuardedRhss _ _) = cont (GuardedRhss an [GuardedRhs an [Qualifier an dots] dots]) Nothing
 
 
-getNames :: Decl -> [String]
+getNames :: Decl_ -> [String]
 getNames x = case x of
     FunBind{} -> name
     PatBind{} -> name
     TypeDecl{} -> name
-    DataDecl _ _ _ _ _ cons _ -> name ++ [fromNamed x | QualConDecl _ _ _ x <- cons, x <- f x]
-    GDataDecl _ _ _ _ _ _ cons _ -> name ++ [fromNamed x | GadtDecl _ x _ <- cons]
+    DataDecl _ _ _ _ cons _ -> name ++ [fromNamed x | QualConDecl _ _ _ x <- cons, x <- f x]
+    GDataDecl _ _ _ _ _ cons _ -> name ++ [fromNamed x | GadtDecl _ x _ <- cons]
     TypeFamDecl{} -> name
     DataFamDecl{} -> name
     ClassDecl{} -> name
@@ -60,9 +67,9 @@ getNames x = case x of
     where
         name = [fromNamed x]
 
-        f (ConDecl x _) = [x]
-        f (InfixConDecl _ x _) = [x]
-        f (RecDecl x ys) = x : concatMap fst ys
+        f (ConDecl _ x _) = [x]
+        f (InfixConDecl _ _ x _) = [x]
+        f (RecDecl _ x ys) = x : concat [y | FieldDecl _ y _ <- ys]
 
 
 suggestName :: String -> Maybe String
@@ -81,6 +88,8 @@ suggestName x = listToMaybe [f x | not $ isSym x || good || "prop_" `isPrefixOf`
         g [] = []
 
 
-replaceNames :: Data a => [(String,String)] -> a -> a
+replaceNames :: Biplate a (Name S) => [(String,String)] -> a -> a
 replaceNames rep = descendBi f
-    where f x = fromMaybe x $ lookup x rep
+    where f (Ident _ x) = Ident an $ fromMaybe x $ lookup x rep
+          f x = x
+
