@@ -2,7 +2,7 @@
 
 module Report(writeReport) where
 
-import Type
+import Idea
 import Control.Arrow
 import Data.List
 import Data.Maybe
@@ -13,18 +13,17 @@ import Paths_hlint
 import Language.Haskell.HsColour.CSS
 
 
-writeTemplate :: [(String,[String])] -> FilePath -> IO ()
-writeTemplate content to = do
-    dat <- getDataDir
-    src <- readFile $ dat </> "report.html"
+writeTemplate :: FilePath -> [(String,[String])] -> FilePath -> IO ()
+writeTemplate dataDir content to = do
+    src <- readFile $ dataDir </> "report.html"
     writeFile to $ unlines $ concatMap f $ lines src
     where
         f ('$':xs) = fromMaybe ['$':xs] $ lookup xs content
         f x = [x]
 
 
-writeReport :: FilePath -> [Idea] -> IO ()
-writeReport file ideas = writeTemplate inner file
+writeReport :: FilePath -> FilePath -> [Idea] -> IO ()
+writeReport dataDir file ideas = writeTemplate dataDir inner file
     where
         generateIds :: [String] -> [(String,Int)] -- sorted by name
         generateIds = map (head &&& length) . group . sort
